<?php
if( !defined( '_VALID_MOS' ) && !defined( '_JEXEC' ) ) die( 'Direct Access to '.basename(__FILE__).' is not allowed.' );
/**
* oanda.com Currency Converter Plugin
* 
* @version $Id: convertOANDA.php 2009-03-12 14:10 martin
*
*  MGS Creativa - Web Intelligent Solutions - Argentina - http://www.mgscreativa.com
*  Martin Sebastian Brilla Ghia - martin@mgscreativa.com
*
* @package VirtueMart
* @subpackage classes
* @copyright Copyright (C) 2009 Martin Sebastian Brilla Ghia - All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
* VirtueMart is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
*
* http://virtuemart.net
*/

/**
 * This class uses the currency rates provided by oanda.com
 */
class convertOANDA {
	
    /********** OPTIONS START **********/

    // Set to true to round up prices IE: $XX.86 -> $XX.99, set to false to leave
    // the exact value of currency server
	var $roundUp = false; 

    // Set the extra charge to add to converted price, sometimes the
    // currency server differs from the local bank currency
    // It has no effect if $roundUp is set to true
    var $extraCharge = 0.00; 

    // Set the cache lifetime in minutes, MAX 1440 (24h)
    // 1h  -> 60
    // 2h  -> 120
    // 6h  -> 360
    // 12h -> 720
    // 18h -> 1080
    // 24h -> 1440
    var $cache_time_update = 360;

    /********** OPTIONS END **********/


    /********** DONT TOUCH ANYTHING BEYOND THIS LINE **********/
	var $archive = true;
	var $suffix   = '_';
	var $service = 'Oanda';
    var $vmVer   = 0;

	/**
	 * Converts an amount from one currency into another using
	 * the oanda.com webpage
	 *
	 * @param float $amountA
	 * @param string $currA defaults to $vendor_currency
	 * @param string $currB defaults to $GLOBALS['product_currency'] (and that defaults to $vendor_currency)
	 * @return the converted amount or the original amount when failure
	 */
	function convert( $amountA, $currA='', $currB='' ) {
        $this->vmVer = $this->getVer();

        if ($this->vmVer<2)
    		global $vendor_currency;

        $this->enqueueMessage('Using convertOANDA.php currency plugin for VM version ' . $this->vmVer);
	
		if( !$currA ) {
			$currA = $vendor_currency;
		}

		if( !$currB ) {
			$currB = $GLOBALS['product_currency'];
		}

		if( $currA == $currB ) {		
			return $amountA;
		}
		
		if ($this->roundUp)		
			$val = ceil((float)$amountA * (float)$this->oandaCurrency($currB,$currA)) - 0.01 ;
		else
			$val = ((float)$amountA * (float)$this->oandaCurrency($currB,$currA)) + $this->extraCharge;

		$this->enqueueMessage('Converted '.$amountA.' '.$currA.' to '.$val.' '.$currB);
		
		return $val;
	}


	// Function to encapsulate Oanda communication and cache handling
	function oandaCurrency($code, $base='') {
        if ($this->vmVer<2)
    		global $mosConfig_cachepath, $mosConfig_absolute_path, $vendor_currency;

		$url  = "http://www.oanda.com/convert/fxdaily?value=1&redirected=1&exch=$code&format=CSV&dest=Get+Table&sel_list=$base";

        $this->enqueueMessage('Will visit this URL if no cache found '.$url);

		// Start cache code		
		setlocale(LC_TIME, "en-GB");
		$now = time();
		if (date("I")) {
			$now += 3600;
		}
		
		$weekday_now_local = gmdate('w', $now);
		$date_now_local = gmdate('Ymd', $now);
		$time_now_local = gmdate('Hi', $now);

		if( is_writable(JPATH_CACHE) ) {
			$store_path = JPATH_CACHE;
		}
		else {
			$store_path = JPATH_ROOT.DS."tmp";
		}
		  
		$archivefile_name = $store_path.'/'.$base.'-'.$code.$this->suffix.$this->service.'.txt';
		$oanda_filename = $url;
		
		if(file_exists($archivefile_name) && filesize( $archivefile_name ) > 0 ) {
		  	$file_datestamp = date('Ymd', filemtime($archivefile_name)); 
		    $file_timestamp = date('Hi', filemtime($archivefile_name)); 
		  	
	    	if( date( 'w' ) > 0 && date( 'w' ) < 6 
	    		&& $file_datestamp != $date_now_local
	    		||  ($time_now_local - $file_timestamp) > $this->cache_time_update) {
		    		$curr_filename = $oanda_filename;
    			    $this->enqueueMessage( "Cache file exists but is too old!, must visit currency server..." );
	    	} else {
    		  	$curr_filename = $archivefile_name;
    		  	$this->archive = false;	
    			$this->enqueueMessage( "Bingo! cached file..." );
	    	}
		} else {
			$curr_filename = $oanda_filename;
			$this->enqueueMessage( "Must visit $this->service..very slow this time!..." );
		}
		  
		if( !is_writable( $store_path )) {
		  $this->archive = false;
		  $this->enqueueMessage( "The file $archivefile_name can't be created. The directory $store_path is not writable" );
		}
		if( $curr_filename == $oanda_filename ) {
            $contents = @file_get_contents( $curr_filename );
			$contents = explode('\n', $contents);
		} else {
			$this->enqueueMessage( "Exchange Ratio $base-$code: " . @file_get_contents( $curr_filename ));
			return @file_get_contents( $curr_filename );
		}
		if( $contents !='' ) {
				$match = array();
                $regexp='/(.+),(\w{3}),([0-9.]+),([0-9.]+)/i';
				preg_match($regexp, implode('', $contents), $match);
				if (sizeof($match) > 0 && $this->archive ) {
					file_put_contents( $archivefile_name, $match[3] );
					$this->enqueueMessage( "Exchange Ratio $base-$code: $match[3]" );
					return $match[3];
				} else {
					return 1;
				}
		} else {
			$this->enqueueMessage( 'Failed to retrieve the currencies.');
			$GLOBALS['product_currency'] = $vendor_currency;
			return 1;
		}
	}

    function enqueueMessage($message) {
		if ($this->vmVer > 1)
            vmdebug($message);
        else {
            global $vmLogger;
            $vmLogger->debug($message);
        }
    }

    function getVer() {
        if (is_dir(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'classes'.DS.'currency')) 
            $ver = 1;
        else if (is_dir(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'plugins'.DS.'currency_converter'))
            $ver = 2;
        else
            $ver = 0;

        return $ver;
    }
}
?>
